/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.figure;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Path;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * This shape consists of an ellipse that is surrounded by transparent insets.
 * It is filled with {@link #getBackgroundColor()}. A light top-down gradient
 * from white to the background color is painted over the background color to
 * achieve a 3 dimensional reflection effect.
 * <p>
 * Anti-aliasing is enabled.
 * 
 * @author hummel
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: CF233E62948846C66408FEE3848CFBDE
 */
public class PrettyInsetEllipse extends Shape {

	/** The transparent insets surrounding this shape. */
	protected final Insets insets;

	/** A complete set of reflections for 3 dimensional ellipses. */
	protected static final Image reflections = AbstractUIPlugin
			.imageDescriptorFromPlugin("edu.tum.cs.ccts.editor.base",
					"icons/ellipse-reflections.png").createImage();

	/** Constructor. */
	public PrettyInsetEllipse(Insets insets) {
		this.insets = insets;
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Paints a rounded rectangle with a top-down gradient from white to
	 * background color.
	 */
	@Override
	protected void fillShape(Graphics gfx) {
		EVisualStyle style = EVisualStyle.get();
		gfx.pushState();
		Rectangle croppedBounds = getCroppedBounds();
		if (style == EVisualStyle.Glossy) {
			// Glossy style.
			gfx.setBackgroundColor(ColorConstants.black);
			gfx.setAntialias(SWT.ON);
			gfx.setAlpha(48);
			gfx.fillOval(croppedBounds.getCopy().translate(5, 5));
			gfx.restoreState();
			if (getBackgroundColor().getGreen() < 255) {
				gfx.setBackgroundColor(ColorConstants.red);
			} else {
				gfx.setBackgroundColor(ColorConstants.orange);
			}
			gfx.fillOval(croppedBounds);
			gfx.setForegroundColor(ColorConstants.yellow);
			gfx.setAlpha(180);

			Path path = new Path(null);
			path.addArc(croppedBounds.x, croppedBounds.y, croppedBounds.width,
					croppedBounds.height, 0, 360);
			gfx.setClip(path);
			gfx.fillGradient(croppedBounds, true);
			gfx.restoreState();
			path.dispose();

			gfx.setAntialias(SWT.ON);
			gfx.setAlpha(220);
			gfx.drawImage(reflections, new Rectangle(reflections.getBounds()),
					croppedBounds);
		} else {
			// Traditional style is default.
			gfx.fillOval(croppedBounds);
			gfx.setForegroundColor(ColorConstants.white);
			gfx.setAlpha(200);

			Path path = new Path(null);
			path.addArc(croppedBounds.x, croppedBounds.y, croppedBounds.width,
					croppedBounds.height, 0, 360);
			gfx.setClip(path);
			gfx.fillGradient(croppedBounds, true);
		}
		gfx.popState();
	}

	/** Returns a copy of the bounds cropped by the insets. */
	protected Rectangle getCroppedBounds() {
		return bounds.getCropped(insets);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Additionally enables anti-aliasing.
	 */
	@Override
	protected void outlineShape(Graphics gfx) {
		EVisualStyle style = EVisualStyle.get();
		gfx.pushState();
		gfx.setAntialias(SWT.ON);
		if (style == EVisualStyle.Glossy) {
			// Glossy style.
			gfx.setLineCap(SWT.CAP_ROUND);
			final int scale = (int) gfx.getAbsoluteScale();
			gfx.setLineWidth(scale + 1);
			Color bg = ColorConstants.orange;
			Color bgdark = new Color(bg.getDevice(), Math.max(0,
					(bg.getRed() - 20)), Math.max(0, (bg.getGreen() - 20)),
					Math.max(0, (bg.getBlue() - 20)));
			gfx.setForegroundColor(bgdark);
			gfx.drawOval(getCroppedBounds());
			bgdark.dispose();
		} else {
			// Traditional style is default.
			gfx.drawOval(getCroppedBounds().resize(-1, -1));
		}
		gfx.popState();
	}
}
